<?php

namespace QUI\CSRF;

use QUI;
use ParagonIE\AntiCSRF\AntiCSRF;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;

use function defined;
use function htmlspecialchars;
use function md5;
use function rand;

use const ENT_QUOTES;

class EventHandler
{
    public static function onTemplateGetHeader(QUI\Template $TemplateManager): void
    {
        $token = md5((string)rand(100000, 999999));
        $index = md5((string)rand(100000, 999999));

        try {
            $csrf = new AntiCSRF();
            $tokenArray = $csrf->getTokenArray();

            if (isset($tokenArray['_CSRF_TOKEN'])) {
                $token = $tokenArray['_CSRF_TOKEN'];
                $token = htmlspecialchars($token, ENT_QUOTES, 'UTF-8');
            }

            if (isset($tokenArray['_CSRF_INDEX'])) {
                $index = $tokenArray['_CSRF_INDEX'];
                $index = htmlspecialchars($index, ENT_QUOTES, 'UTF-8');
            }
        } catch (\Exception $Exception) {
            QUI\System\Log::addError($Exception->getMessage(), [
                'type' => 'csrf token creation'
            ]);
        }

        $TemplateManager->extendHeader(
            '
            <meta content="' . $token . '" name="csrf-token" data-no-cache="1" />
            <meta content="' . $index . '" name="csrf-index" data-no-cache="1" />
            <script src="' . URL_OPT_DIR . 'quiqqer/csrf/bin/csrf.js"></script>
            '
        );
    }

    public static function onHeaderLoaded(): void
    {
        if (php_sapi_name() !== 'cli') {
            return;
        }

        if (defined('QUIQQER_CONSOLE') && QUIQQER_CONSOLE) {
            return;
        }

        try {
            $csrf = new AntiCSRF();
        } catch (\Error $e) {
            QUI\System\Log::addError('CSRF token error ' . $e->getMessage());
            return;
        }

        if (defined('QUIQQER_AJAX')) {
            $csrf->validateRequestOrThrow();
        }

        if (!empty($_POST)) {
            $csrf->validateRequestOrThrow();
        }
    }

    public static function onRestInit(QUI\REST\Server $Server): void
    {
        $Slim = $Server->getSlim();

        $csrfMiddleware = function (Request $request, RequestHandler $handler) use ($Slim) {
            $path = $request->getUri()->getPath();

            if (!RestRegistrar::mustPathBeChecked($path)) {
                return $handler->handle($request);
            }

            if ($request->getMethod() === 'GET' && RestRegistrar::isCsrfProtectedGetRoute($path)) {
                $csrfToken = $request->getHeader('X-CSRF-TOKEN')[0] ?? '';
                $csrfIndex = $request->getHeader('X-CSRF-INDEX')[0] ?? '';

                if (
                    !isset($_SESSION['CSRF'][$csrfIndex])
                    && $_SESSION['CSRF'][$csrfIndex] === $csrfToken
                ) {
                    $response = $Slim->getResponseFactory()->createResponse();
                    $response->getBody()->write('Invalid CSRF token!');
                    return $response->withStatus(401);
                }

                return $handler->handle($request);
            }


            if (in_array($request->getMethod(), ['POST', 'PUT', 'DELETE'])) {
                $csrf = new AntiCSRF();

                if (!$csrf->validateRequest()) {
                    $response = $Slim->getResponseFactory()->createResponse();
                    $response->getBody()->write('Invalid CSRF token!');
                    return $response->withStatus(401);
                }
            }

            return $handler->handle($request);
        };

        $Slim->add($csrfMiddleware);
    }
}
