<?php

namespace QUI\CSRF;

use QUI;

use function fnmatch;
use function is_string;
use function ltrim;
use function rtrim;

class RestRegistrar
{
    /**
     * @var string[]
     */
    protected static array $routes = [];

    /**
     * @var string[]
     */
    protected static array $getRoutes = [];

    public static function register(string $route): void
    {
        self::$routes[] = $route;

        // add base path
        try {
            $Package = QUI::getPackage('quiqqer/rest');
            $Config = $Package->getConfig();
            $basePath = $Config?->getValue('general', 'basePath');

            if (!is_string($basePath)) {
                $basePath = '';
            }

            $basePath = rtrim($basePath, '/');
            $basePath = $basePath . '/';

            self::$routes[] = $basePath . ltrim($route, '/');
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::addDebug($Exception->getMessage());
        }
    }

    /**
     * @return string[]
     */
    public static function getRoutes(): array
    {
        return self::$routes;
    }

    public static function mustPathBeChecked(string $route): bool
    {
        foreach (self::$routes as $pathPattern) {
            if (fnmatch($pathPattern, $route)) {
                return true;
            }
        }

        return false;
    }


    public static function includeGetRoutes(string $route): void
    {
        self::$getRoutes[] = $route;

        // add base path
        try {
            $Package = QUI::getPackage('quiqqer/rest');
            $Config = $Package->getConfig();
            $basePath = $Config?->getValue('general', 'basePath');

            if (!is_string($basePath)) {
                $basePath = '';
            }

            $basePath = rtrim($basePath, '/');
            $basePath = $basePath . '/';

            self::$getRoutes[] = $basePath . ltrim($route, '/');
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::addDebug($Exception->getMessage());
        }
    }

    /**
     * @return string[]
     */
    public static function getGetRoutes(): array
    {
        return self::$getRoutes;
    }

    public static function isCsrfProtectedGetRoute(string $route): bool
    {
        foreach (self::$getRoutes as $pathPattern) {
            if (fnmatch($pathPattern, $route)) {
                return true;
            }
        }

        return false;
    }
}
